@@ -195,7 +195,7 @@ def stack_magic(stack):
             if '__builtins__' in fl:
                 #print('globals found at', i)
                 return fl
-    elif in_notebook or in_ipython or in_test:
+    elif in_notebook or in_ipython or in_test or '_run_module_as_main' in function_names:
         index = 1  # this seems to work for now
     elif function_names.count('main') >= 2:
         # FIXME this is a hack that only works
@@ -355,14 +355,18 @@ def refile(script__file__, path):
     return str(Path(script__file__).parent / path)
 
 
-def relative_path(script__file__):
+def relative_path(script__file__, no_wd_value=None):
     # FIXME will break outside of subfolders of working_dir neuron_models folder ...
     working_dir = get_working_dir(script__file__)
+    if working_dir is None and no_wd_value is not None:
+        return no_wd_value
+
     rpath = (Path(script__file__).
              resolve().
              relative_to(working_dir.
                          resolve()).
              as_posix())
+
     return rpath
 
 
